@@ -100,15 +100,15 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 // SYN requests nearer than RECENT_DROP_TIME are considered parts of the
 // same attempt to connect and given the same block response (seconds)
-#if !defined RECENT_DROP_TIME
-#define RECENT_DROP_TIME 10
+#if !defined IPQBDB_RECENT_DROP_TIME
+#define IPQBDB_RECENT_DROP_TIME 10
 #endif
 
 // The UPDATE_TICK_TIME is the minimal amount of time to consider when
 // rehabilitating an ip. Since the probability is rounded to an integer,
 // this should be large enough to make rounding errors small. (seconds)
-#if !defined UPDATE_TICK_TIME
-#define UPDATE_TICK_TIME 20
+#if !defined IPQBDB_UPDATE_TICK_TIME
+#define IPQBDB_UPDATE_TICK_TIME 20
 #endif
 
 // The decay is the time it takes for the probability to halve, 6 hours.
