@@ -2,7 +2,7 @@
 * dbstruct.h
 
 
-Copyright (C) 2008-2011 Alessandro Vesely
+Copyright (C) 2008-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -111,7 +111,7 @@ int set_descr_cnt(DB *db1, int descr_id, int is_new_record,
 int open_descrdb(char *fname, DB_ENV *db_env, DB** db1, DB** db2);
 int check_whitelist(DB *, uint32_t ip, double *decay);
 
-typedef void (*write_block_cb)(uint32_t ip_addr,
+typedef void (*write_block_cb)(uint32_t ip_addr, int reason_id,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db_d);
 enum
 { write_block_force_decay = 1, write_block_force_probability = 2,
