@@ -164,13 +164,13 @@ static struct poptOption opttab[] =
 	POPT_TABLEEND
 };
 
-static int get_ip_address(char *ip_address, uint32_t* ip_addr)
+static int get_ip_address(char *an_ip_address, uint32_t* ip_addr)
 {
 	struct in_addr in;
-	int rtc = my_inet_pton(ip_address, &in);
+	int rtc = my_inet_pton(an_ip_address, &in);
 	if (rtc == 0 || in.s_addr == INADDR_ANY) // 0 is not valid
 	{
-		report_error(&ap, LOG_ERR, "invalid ip_address %s\n", ip_address);
+		report_error(&ap, LOG_ERR, "invalid ip_address %s\n", an_ip_address);
 		return 1;
 	}
 
@@ -182,8 +182,9 @@ static int get_all_ip_address(uint32_t **ip_addr)
 {
 	size_t count = 0;
 	char **a = ip_address;
-	while (*a++)
-		++count;
+	if (a && *a)
+		while (*a++)
+			++count;
 
 	if (count == 0)
 	{
@@ -274,12 +275,12 @@ static void tab_printtime(char const *hdr, time_t ref, time_t now)
 	printf("\t%-15s %18s %29s\n", hdr, buf2, buf);
 }
 
-static void display_written_block(uint32_t ip_addr,
+static void display_written_block(uint32_t ip_addr, int reason_id,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db_d)
 {
 	// if syslog_opt, log a line anyway (unlikely given -v).
 	if (ap.mode)
-		logline_written_block(ip_addr, ip_data, old_data, db_d);
+		logline_written_block(ip_addr, reason_id, ip_data, old_data, db_d);
 
 	struct in_addr in;
 	char descr[19 /* IPQBDB_DESCR_MAX_LENGTH */];
@@ -360,12 +361,12 @@ write_block_display(DB *db, uint32_t ip_addr, double decay, int reason_id, DB *d
 
 static void run_exec_ip(connkill_cmd *cmd, uint32_t ip_addr)
 {
-	char *ip_address;
+	char *my_ip_address;
 	struct in_addr in;
 	memcpy(&in, &ip_addr, sizeof in);
 
-	ip_address = strdup(inet_ntoa(in));
-	run_exec(cmd, ip_address, &ap);
+	my_ip_address = strdup(inet_ntoa(in));
+	run_exec(cmd, my_ip_address, &ap);
 }
 
 int main(int argc, char const *argv[])
