@@ -3,7 +3,7 @@
 * written by ale in milano on 15 nov 2008
 * delete and/or list records in a block.db
 
-Copyright (C) 2008-2011 Alessandro Vesely
+Copyright (C) 2008-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -33,12 +33,9 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <syslog.h>
 #include <errno.h>
 #include <locale.h>
-
-#define __USE_BSD 1
-// for u_int in db.h
 #include <sys/types.h>
 
-// Berkeley DB v4.4
+// Berkeley DB v5.3
 #include <db.h>
 
 #include <popt.h>
@@ -171,9 +168,9 @@ static int init_histog(histog *h, time_t secs)
 	memset(h, 0, sizeof *h);
 	if ((h->his = (int*)malloc(HISLOG_SIZE * sizeof(int))) == NULL)
 		return -1;
-	
+
 	memset(h->his, 0, HISLOG_SIZE * sizeof(int));
-	
+
 	if (secs <= 0) secs = 86400;
 	else
 	{
@@ -181,20 +178,20 @@ static int init_histog(histog *h, time_t secs)
 		if (secs < 3600)
 			secs = 3600;
 	}
-	
+
 	double const period = (double)secs;
-   int mesh = (int)floor((double)HISLOG_SIZE /
-   	(1.0 + log((double)((365U*4U + 1U)*86400U)/period)));
+	int mesh = (int)floor((double)HISLOG_SIZE /
+		(1.0 + log((double)((365U*4U + 1U)*86400U)/period)));
 
-   if (mesh < 20) mesh = 20;
-   else if (mesh > HISLOG_SIZE/2) mesh = HISLOG_SIZE/2;
+	if (mesh < 20) mesh = 20;
+	else if (mesh > HISLOG_SIZE/2) mesh = HISLOG_SIZE/2;
+
+	h->mesh = mesh;
+	h->dmesh = (double)mesh;
+	h->period = secs;
+	h->k = M_E/period;
+	h->log_k = 1.0 - log(period);
 
-   h->mesh = mesh;
-   h->dmesh = (double)mesh;
-   h->period = secs;
-   h->k = M_E/period;
-   h->log_k = 1.0 - log(period);
-   
 	return 0;
 }
 
@@ -219,7 +216,7 @@ static char const *histog_index(histog const *h, time_t t, unsigned *result)
 		double dndx = floor(histog_index_d(h, (double)t));
 		if (!isnormal(dndx) || dndx > 10000.0 || dndx < 0.0)
 			return "unnormal or too big time";
-	
+
 		*result = (unsigned)dndx;
 	}
 
@@ -237,7 +234,7 @@ static char const *histog_time(histog const *h, int ndx, time_t *result)
 	{
 		double const i = (double)ndx;
 		double tt = exp((ndx)/(h->dmesh))/h->k;
-		
+
 		/*
 		* Find the smallest (more recent, when mapped to real time) side of
 		* the interval whose t values are mapped to ndx. Note that the interval
@@ -249,16 +246,16 @@ static char const *histog_time(histog const *h, int ndx, time_t *result)
 		double ii = histog_index_d(h, tt);
 		double t = ((i - ii)/h->dmesh + 1.0) * tt;
 		// printf("  for %d: %g->%g", ndx, tt, t);
-		
+
 		while (isnormal(t) && fabs(t - tt) > 0.4)
 		{
 			tt = t;
 			ii = histog_index_d(h, tt);
 			t = ((i - ii)/h->dmesh + 1.0) * tt;
-			// printf("->%g", t);			
+			// printf("->%g", t);
 		}
 		// putchar('\n');
-		
+
 		int fp = fpclassify(t);
 		if (fp == FP_NORMAL)
 			// since we sought the boundary, rounding can go on either side,
@@ -267,7 +264,7 @@ static char const *histog_time(histog const *h, int ndx, time_t *result)
 		else
 			return "non-normal or too big";
 	}
-	
+
 	return NULL;
 }
 
@@ -279,7 +276,7 @@ static const char* count_histog(histog *h, time_t secs)
 	char const *err = histog_index(h, secs, &ndx);
 	if (err)
 		return err;
-	
+
 	if (ndx >= HISLOG_SIZE)
 		h->out_of_range += 1;
 	else
@@ -295,7 +292,7 @@ static const char *increase_secs(histog *h, int target, time_t secs)
 */
 {
 	assert(h && h->increase == 0);
-	
+
 #if defined NDEBUG
 	(void)secs; // only used  to compute intervals
 #else
@@ -315,11 +312,11 @@ static const char *increase_secs(histog *h, int target, time_t secs)
 				char const *er_f, *er_t = NULL;
 				char from[TIME_STRING_BUFSIZE], to[TIME_STRING_BUFSIZE];
 				time_t from_time, to_time, mark = h->now - secs;
-				
+
 				er_f = histog_time(h, i + 1, &from_time);
 				if (er_f == NULL)
 					er_f = time_string(mark - from_time + 1, h->now, from);
-				
+
 				er_t = histog_time(h, i, &to_time);
 				if (er_t == NULL)
 					er_t = time_string(mark - to_time, h->now, to);
@@ -432,7 +429,7 @@ static long max_records = 0;
 static int version_opt, help_opt, syslog_opt, cleanup_opt;
 static int only_blocked, non_blocked;
 static int verbose = -1;
-static int list_opt, raw_opt, del_opt, ban_opt, stat_opt;
+static int list_opt, raw_opt, del_opt, ban_opt, stat_opt, simple_opt;
 static struct poptOption opttab[] =
 {
 	{"only-blocked", '\0', POPT_ARG_NONE, &only_blocked, 0,
@@ -459,6 +456,8 @@ static struct poptOption opttab[] =
 	"List according to ibd-ban format", NULL},
 	{"ls-raw", '\0', POPT_ARG_NONE, &raw_opt, 0,
 	"List selected records displaying plain numbers", NULL},
+	{"ls-simple", '\0', POPT_ARG_NONE, &simple_opt, 0,
+	"List selected records in a simple fashion", NULL},
 	{"del", '\0', POPT_ARG_NONE, &del_opt, 0,
 	"Delete selected records", NULL},
 	{"stats", 'S', POPT_ARG_NONE, &stat_opt, 0,
@@ -540,7 +539,7 @@ static int all_reason_id(DB *db1, DB *db2)
 {
 	if (reason_string == NULL || *reason_string == 0)
 		return 0;
-	
+
 	char *t = NULL;
 	unsigned long l = strtoul(reason_string, &t, 0);
 	if (t && *t == 0 && l < INT_MAX)
@@ -711,7 +710,7 @@ static int get_probability(int *prob, char *probability)
 		l /= 100;
 		++t;
 	}
-	
+
 	l /= deno;
 
 	if (t == NULL || *t != 0 || l > RAND_MAX)
@@ -738,6 +737,7 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t *secs, int *prob, histog *his)
 /*
 * If his != NULL, compute the histogram of records exceeding the given min-time
 * and perform no other operation.
+* Despite its name, this function also does simple listing of the block database.
 */
 {
 	DBC *curs;
@@ -768,7 +768,7 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t *secs, int *prob, histog *his)
 		report_error(&ap, LOG_ERR, "memory fault\n");
 		return -1;
 	}
-	
+
 	/*
 	* acquire a read cursor: this can possibly delete a record that has
 	* been modified since we read it, even if it was requested a min
@@ -846,9 +846,9 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t *secs, int *prob, histog *his)
 			if (selected)
 			{
 				if (adjusted_prob > 0 &&
-					UPDATE_TICK_TIME < delta &&
+					IPQBDB_UPDATE_TICK_TIME < delta &&
 					ip_data->decay > 0.0)
-						adjusted_prob = 
+						adjusted_prob =
 							rehabilitated_prob(adjusted_prob, delta, ip_data->decay);
 
 				// adjusted prob should never be negative, but just in case...
@@ -955,6 +955,13 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t *secs, int *prob, histog *his)
 							ip_data->decay,
 							descr);
 				}
+				else if (simple_opt)
+					printf("%s was caught %d time%s since %s",
+						inet_ntop(AF_INET, (struct in_addr*)&cur_key,
+							ntopbuf, sizeof ntopbuf),
+						ip_data->caught_cnt,
+						ip_data->caught_cnt > 1? "s": "",
+						ctime(&ip_data->created)/* ctime ends with \n */);
 
 				total_selected += 1;
 			}
@@ -1020,7 +1027,7 @@ static int do_stat(DB *db)
 	key.flags = data.flags = DB_DBT_USERMEM;
 	data.data = &u;
 	data.ulen = data.size = sizeof u;
-	
+
 	rtc = db->cursor(db, NULL, &curs, 0);
 	if (rtc)
 	{
@@ -1065,7 +1072,7 @@ static int do_stat(DB *db)
 			decay_add_cnt += u.drec.decay_add_cnt;
 		}
 	}
-	
+
 	curs->close(curs);
 
 	if (rtc != DB_NOTFOUND)
@@ -1073,13 +1080,14 @@ static int do_stat(DB *db)
 	else
 		rtc = 0;
 
-	printf("%'11" PRIi64 " %'11" PRIi64 " %'11" PRIi64
-		" %8s %8s" " %'11" PRIi64 "\n",
-		caught_cnt, block_cnt, rec_cnt,
-		"",
-		"", decay_add_cnt);
+	if (total_selected > 1)
+		printf("%'11" PRIi64 " %'11" PRIi64 " %'11" PRIi64
+			" %8s %8s" " %'11" PRIi64 " %8s Total\n",
+			caught_cnt, block_cnt, rec_cnt,
+			"",
+			"", decay_add_cnt, "");
 
-	return rtc;	
+	return rtc;
 }
 
 int main(int argc, char const *argv[])
@@ -1141,11 +1149,11 @@ int main(int argc, char const *argv[])
 			if (raw_opt)
 				list_opt = 1;
 
-			if (!list_opt && !del_opt && !ban_opt && !stat_opt && errs == 0)
+			if (!list_opt && !del_opt && !ban_opt && !stat_opt && !simple_opt && errs == 0)
 			{
 				fprintf(stdout,
 					"%s: at least one of"
-					" --ls, --ls-raw, --ls-ban, --stats, or --del"
+					" --ls, --ls-raw, --ls-ban, ls-simple, --stats, or --del"
 					" must be specified\n",
 					err_prefix);
 				errs = 1;
@@ -1266,7 +1274,7 @@ int main(int argc, char const *argv[])
 										else
 											printf(
 												" min age increased by %jd before %s!\n",
-													(intmax_t)his.increase, before);										
+													(intmax_t)his.increase, before);
 									}
 									else
 										printf(" min age did not vary!\n");
@@ -1290,7 +1298,7 @@ int main(int argc, char const *argv[])
 			}
 
 			if (rtc == 0 && caught_signal == 0 &&
-				(list_opt || raw_opt || ban_opt || del_opt))
+				(list_opt || raw_opt || ban_opt || del_opt || simple_opt))
 			{
 				if (verbose)
 				{
@@ -1299,7 +1307,7 @@ int main(int argc, char const *argv[])
 						" min prob %d=%.2f%%, max prob %d=%.2f%%.\n",
 						del_opt? "delete": "",
 						del_opt && (list_opt || raw_opt || ban_opt)? " and ": "",
-						(list_opt || raw_opt || ban_opt)? "list": "",
+						(list_opt || raw_opt || ban_opt || simple_opt)? "list": "",
 						((unsigned char*)&ip.first)[0],
 						((unsigned char*)&ip.first)[1],
 						((unsigned char*)&ip.first)[2],
@@ -1316,7 +1324,7 @@ int main(int argc, char const *argv[])
 				}
 				rtc = do_del(db_block, db_de1, &ip, secs, prob, NULL);
 			}
-			
+
 			if (rtc == 0 && caught_signal == 0 && stat_opt)
 			{
 				rtc = do_stat(db_de1);
