@@ -3,7 +3,7 @@
 * written by ale in milano on 28 sep 2008
 * read from a socket and extract pcre patterns to add to block.db
 
-Copyright (C) 2008-2011 Alessandro Vesely
+Copyright (C) 2008-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -46,7 +46,7 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-// Berkeley DB v4.4
+// Berkeley DB v5.3
 #include <db.h>
 
 #include <popt.h>
@@ -72,6 +72,15 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <assert.h>
 
+static const char *month_abbr(unsigned ndx)
+{
+	static const char *abbr[] =
+	{"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+	 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+	if (ndx >= sizeof abbr / sizeof abbr[0]) return "---";
+	return abbr[ndx];
+}
+
 static char *trim(char *s, char *end)
 {
 	while (s < end && isspace(*(unsigned char*)s))
@@ -122,6 +131,8 @@ static char *next_field(char **ps)
 
 static app_private ap;
 static char const err_prefix[] = "ibd-parse";
+#include "logline_written_block.h"
+
 int caught_signal;
 
 #if !defined NDEBUG
@@ -218,7 +229,7 @@ static int setsigs(void)
 	return rtc;
 }
 
-static int initial_count = 5;
+static int initial_count = IPQBDB_INITIAL_COUNT;
 static double initial_decay = IPQBDB_INITIAL_DECAY;
 static char *socket_name = IPQBDB_PARSE_SOCKET;
 static char *db_block_name = IPQBDB_DATABASE_NAME;
@@ -297,7 +308,10 @@ typedef struct initialization_stuff
 	int line;
 
 	int default_ini_prob;
-	int verbose_header_given;
+	unsigned int verbose_header_given:1;
+	unsigned int run_on_terminal:1;
+	unsigned int not_used:30;
+
 } initialization_stuff;
 
 typedef struct runtime_stuff
@@ -316,6 +330,8 @@ typedef struct runtime_stuff
 
 	size_t db_count; // number of entries in db
 
+	time_t start;
+
 } runtime_stuff;
 
 typedef struct socket_stuff
@@ -478,6 +494,9 @@ static int add_fname(initialization_stuff *ini, char *fname)
 
 static int insert_pcre(initialization_stuff *ini,
 	char *re, char *ip, char *reason, char *count, char *fname, char *decay)
+//
+// insert_pcre actually creates the re_list element and inserts it.
+// if verbose and running on terminal, details are printed.
 {
 	int errs = 0, rc;
 	char *t = NULL;
@@ -497,7 +516,7 @@ static int insert_pcre(initialization_stuff *ini,
 	if (ip[0] == '*' && ip[1] == 0)
 	{
 		static const char host_repl[] = "<HOST>";
-		static const char host_expr[] = "(?P<IP>([0-9]{1,3}\\.){3}[0-9]{1,3})";
+		static const char host_expr[] = "[0fF:]*(?P<IP>(:?[0-9]{1,3}\\.){3}[0-9]{1,3})";
 		char *repl = strstr(re, host_repl);
 		if (repl == NULL || strstr(repl + 1, host_repl) != NULL)
 		{
@@ -549,7 +568,7 @@ static int insert_pcre(initialization_stuff *ini,
 			report_error(&ap, LOG_ERR, "info=%d\n", rc);
 			errs |= 4;
 		}
-		else if (verbose && ap.mode == error_report_stderr)
+		else if (verbose && ini->run_on_terminal)
 		{
 			int backrefmax, firstbyte, lastliteral;
 			size_t size, studysize;
@@ -652,7 +671,7 @@ static int insert_pcre(initialization_stuff *ini,
 		if (ini->inited > 0)
 		{
 			if (get_reason_id(&r.reason, reason, &ap,
-				ini->db1, ini->db2, verbose) != 0)
+				ini->db1, ini->db2, verbose && ini->run_on_terminal) != 0)
 					errs |= 2;
 		}
 		else
@@ -828,6 +847,11 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 			initial_count);
 	ini.default_ini_prob += 1; // rounded
 
+	// are we going to daemonize?
+	// daemonize_if_ok can be 0 if called after SIGHUP, so check -D
+	if (no_daemon_opt || no_run_opt)
+		ini.run_on_terminal = 1;
+
 	if (fp == NULL)
 	{
 		report_error(&ap, LOG_CRIT, "cannot open %s: %s\n",
@@ -949,6 +973,8 @@ static int read_pcre(int daemonize_if_ok, runtime_stuff *run)
 		assert(r->decay > 0.0);
 	}
 
+	time(&run->start);
+
 	free(ini.db_fnames);
 	return errs;
 }
@@ -1016,23 +1042,6 @@ static int get_ip_address(uint32_t* ip, char *dest, size_t dsize,
 	return 0;
 }
 
-static void display_written_block(uint32_t ip_addr,
-	ip_data_t *ip_data, ip_data_t *old_data, DB *is_null)
-{
-	(void)is_null; // is null
-
-	struct in_addr in;
-	memcpy(&in, &ip_addr, sizeof in);
-
-	report_error(&ap, LOG_INFO,
-		"%s record for %s, decay: %g, probability: %d=%.2f%%\n",
-		old_data? "changed": "inserted",
-		inet_ntoa(in),
-		ip_data->decay,
-		ip_data->probability,
-		percent_prob(ip_data->probability));
-}
-
 static void spawn_exec(char *ip, connkill_cmd *cmd)
 {
 	pid_t pid = fork();
@@ -1131,7 +1140,7 @@ static int block_matched(runtime_stuff *run, re_list_t const * const r,
 			int probability = r->init_prob;
 			rtc = write_block(run->db[r->db_ndx], ip_addr, decay,
 				r->reason, &probability, 0,
-				verbose? display_written_block: NULL, run->db1);
+				verbose? logline_written_block: NULL, run->db1);
 
 			if (rtc != 0)
 				err = 1;
@@ -1203,6 +1212,33 @@ static int run_pcre(runtime_stuff *run, char const *str, size_t const len)
 
 static void log_re_list(runtime_stuff *run)
 {
+	time_t now = time(NULL);
+	unsigned long elapsed = now - run->start;
+	unsigned dd, hh, mm;
+	char buf[32], *p = &buf[0];
+
+	dd = elapsed / (24UL * 3600UL);
+	if (dd)
+	{
+		elapsed -= dd * 24UL * 3600UL;
+		p += sprintf(p, "%u day(s) ", dd);
+	}
+	hh = elapsed / 3600UL;
+	if (hh || dd)
+	{
+		elapsed -= hh * 3600UL;
+		p += sprintf(p, "%uh ", hh);
+	}
+	mm = elapsed / 60UL;
+	elapsed -= mm * 60UL;
+	sprintf(p, "%um ", mm);
+
+	struct tm tm = *localtime(&run->start);
+	report_error(&ap, LOG_INFO,
+		"stats since %d %s %d (%s%us ago)\n",
+		tm.tm_mday, month_abbr(tm.tm_mon), tm.tm_year + 1900,
+		buf, (unsigned)elapsed);
+
 	re_list_t *r = run->re_list;
 	report_error(&ap, LOG_INFO,
 		"%5.5s %5.5s %5.5s %5.5s %5.5s %19.19s %6.6s %5.5s\n",
@@ -1395,6 +1431,7 @@ int main(int argc, char const *argv[])
 			pcre_config(PCRE_CONFIG_NEWLINE, &newl);
 			fprintf(stdout,
 				"%s: version %s\n"
+				DB_VERSION_STRING "\n"
 				"pcre %s%s%s, with newline %s\n",
 				err_prefix, PACKAGE_VERSION,
 				pcre_version(),
