@@ -29,14 +29,14 @@ Must be included after those definitions.
 This is the line logging for ibd-parse -v and ibd-ban -l.
 */
 
-static void logline_written_block(uint32_t ip_addr,
+static void logline_written_block(uint32_t ip_addr, int reason_id,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db1)
 {
 	struct in_addr in;
 	memcpy(&in, &ip_addr, sizeof in);
 
 	char descr[IPQBDB_DESCR_MAX_LENGTH];
-	get_descr(db1, ip_data->reason_id, 0, descr, sizeof descr);
+	get_descr(db1, reason_id, 0, descr, sizeof descr);
 
 	report_error(&ap, LOG_INFO,
 		"%s record for %s, decay: %g, prob: %.2f%%, %s\n",
