@@ -564,6 +564,10 @@ int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 			rtc = 1;
 		else
 		{
+			/*
+			* Record found, double probability, apply worsening logic,
+			* set the reason to the "most severe" one.
+			*/
 			memcpy(&old_data, &ip_data, sizeof old_data);
 			old = 1;
 			if (ip_data.decay < decay || (force & write_block_force_decay) != 0)
@@ -632,7 +636,8 @@ int write_block(DB *db, uint32_t ip_addr, double decay, int reason_id,
 			set_descr_cnt(db_d, reason_id, !old, NULL, &now);
 			
 		if (verbose_fn)
-			verbose_fn(ip_addr, &ip_data, old? &old_data: NULL, db_d);
+			verbose_fn(ip_addr, reason_id,
+				&ip_data, old? &old_data: NULL, db_d);
 	}
 	
 	return rtc || rtc2;
