@@ -1,7 +1,7 @@
 /*
 * rehabilitated_prob.h
 
-Copyright (C) 2011 Alessandro Vesely
+Copyright (C) 2011-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -32,7 +32,7 @@ rehabilitated_prob(unsigned int prob, time_t delta, double decay)
 * Return the new probability, rehabilitated according to arguments
 */
 {
-	assert(delta > UPDATE_TICK_TIME);
+	assert(delta > IPQBDB_UPDATE_TICK_TIME);
 	assert(decay > 0.0);
 
 	double d = (double)prob * exp2(-(double)delta / decay);
